//+------------------------------------------------------------------+
//|                                                Swiss Army EA.mq4 |
//|                   "It does everything but place its own orders!" |
//+------------------------------------------------------------------+
//|                                    Copyright  2007, Ryan Klefas |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2007, Ryan Klefas (Base 1.8)"
#property link      "rklefas@inbox.com"



extern string    id="==== Identity Settings ====================";
extern bool      ManageBy_Symbol=false;         // If true, EA will only manage order that have matching
                                                //     symbols
extern bool      ManageBy_Magic=false;          // If true, EA will only manage order that have matching
                                                //     magic numbers
                                                // If both are false, EA will manage ALL orders regardless 
                                                //     of magic numbers or symbols
extern int       MagicNumber=999999;            // Magic number EA will try to manage
                                                
                                                
extern string    manage="==== Stop Management ====================";
extern int       BreakEvenAt=0;                 // Set Stoploss to open order price at X pips profit
extern int       BreakEvenSlide=0;              // Move the breakeven point up or down around
                                                //     the order open price
extern int       TrailingStop=0;                // Stoploss follows behind current price by X pips
extern bool      OnlyTrailAfterProfit=false;    // Trailing Stop will only trail when order is profitable


extern string    close="==== Close-out Conditions ====================";
extern bool      ImmediateCloseOut=false;       // Close-out immediately occurs

extern bool      CloseByTime=true;              // Close-out at the specified time
extern int       CloseHour=23;                  // Hour to close
extern int       CloseMinute=55;                // Minute to close

extern bool      CloseByTotalProfit=false;      // Close-out when total profits surpass specified level
extern int       TotalProfit=100;               // Profit in dollars to close

extern bool      CloseByTotalLoss=false;        // Close-out when total losses surpass specified level
extern int       TotalLoss=100;                 // Losses in dollars to close

extern bool      CloseByDrawdown=false;         // Close-out if max percentage drawdown is reached
extern int       MaxDrawdown=25;                // Maximum percentage drawdown allowed


extern string    action="==== Close-out Actions ====================";
extern bool      CloseBuys=false;               // All active buy orders will close
extern bool      CloseSells=false;              // All active sell orders will close
extern bool      DeletePendings=false;          // All pending orders will close
extern bool      CloseEverything=true;          // All orders will close


extern string    extra="==== Extra Settings ====================";
extern string    ExpertName="Swiss Army EA";    // Expert name: for aesthetic purposes
extern bool      Disable_Comments=false;        // EA will not display comments on screen
extern int       Slippage=3;                    // Slippage on closing orders


//+------------------------------------------------------------------+
//| global variables                                              |
//+------------------------------------------------------------------+

       // Individual order types

   int longActive=0,  shortActive=0;
   int longStop=0,    shortStop=0;
   int longLimit=0,   shortLimit=0;
   
       // Accumulative order types
   
   int longPending=0, shortPending=0;
   int allLong=0,     allShort=0;
   int allActive=0,   allPending=0;
   int grandTotal=0;   
   
   
   double actualProfitMonitor=0;
   

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
   onScreenComment(91);
   return(0);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
   onScreenComment(99);
   return(0);
}
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
   
   
   findMyOrders();

   if (allActive>0)
   {
      if (BreakEvenAt>0)
         breakEvenManager();
      if (TrailingStop>0)
         trailingStopManager();
   }
   

   onScreenComment(98);
   

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

   if (ImmediateCloseOut)
      actionFunction();
      
   
   if (CloseByTime)
   {
      if (Hour()==CloseHour && Minute()==CloseMinute)
         actionFunction();
   }

   if (CloseByTotalProfit)
   {
      if (actualProfitMonitor>TotalProfit)
         actionFunction();
   }
   
   if (CloseByTotalLoss)
   {
      if ( actualProfitMonitor < (TotalLoss * (-1)) )
         actionFunction();
   }
   
   if (CloseByDrawdown)
   {
      if (drawdown()>MaxDrawdown)
         actionFunction();
   }
   

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

      
   return(0);  // End of start function
}
  
//+------------------------------------------------------------------+
//| middle-man modules                                               |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+

void closeLongOrder() // Base Version 1.8
{ versatileOrderCloser(1); }

//+------------------------------------------------------------------+

void closeShortOrder() // Base Version 1.8
{ versatileOrderCloser(2); }

//+------------------------------------------------------------------+

void deletePending() // Base Version 1.8
{ versatileOrderCloser(3); }

//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| standardized modules and functions                               |
//+------------------------------------------------------------------+

void onScreenComment(int event) 
{

   if (Disable_Comments==false)
   {
      
      string draw = DoubleToStr(drawdown(), 2);
      string prof = DoubleToStr(actualProfitMonitor, 2);
      
      string header = ExpertName + " run-time statistics: \n\n";
      string taker, stopper;
                      
      string ordercount = "\n" + "\n" +
             "Buy Active:  " + longActive + "\n" +
             "Sell Active:  " + shortActive + "\n" +
             "Buy Stop:  " + longStop +  "\n" +
             "Sell Stop:  " + shortStop + "\n" +
             "Buy Limit:  " + longLimit + "\n" +
             "Short Limit:  " + shortLimit + "\n" +
             "Grand Total:  " + grandTotal
             ;
                 
      string stats = 
             "Total Profit/Loss:  " + prof + "\n" +
             "Drawdown:  "          + draw + "%\n"
             ;
      
      
      if (!TrailingStop>0 && !BreakEvenAt>0)
         stopper = "Stop management disabled\n";
      else if (TrailingStop>0 && BreakEvenAt>0)
         stopper = "Trailing Stop and Breakeven management enabled\n";
      else if (TrailingStop>0)
         stopper = "Trailing Stop management enabled\n";
      else if (BreakEvenAt>0)
         stopper = "Breakeven management enabled\n";



      if (ManageBy_Magic==false && ManageBy_Symbol==false)
         taker = "Managing ALL orders in this terminal\n";
      else if (ManageBy_Magic && ManageBy_Symbol)
         taker = "Managing only orders that have magic number " + MagicNumber + " and of symbol " + Symbol() + "\n";     
      else if (ManageBy_Magic)
         taker = "Managing only orders that have magic number " + MagicNumber + "\n";     
      else if (ManageBy_Symbol)
         taker = "Managing only orders of symbol " + Symbol() + "\n";     



      switch (event)
      {

         case 91: Comment(ExpertName + " is waiting for the next tick to begin."); break;
         case 98: Comment(header + taker + stats + stopper + ordercount); break;
         case 99: Comment(" "); break;

      }
   }

}

//+------------------------------------------------------------------+

int simpleMagicGenerator() 
{

   return (MagicNumber);
}

//+------------------------------------------------------------------+

bool orderBelongsToMe()
{
   
   bool magicPermission=false;
   bool symbolPermission=false;
   
   
   if (ManageBy_Magic)
   {
      if (OrderMagicNumber() == simpleMagicGenerator())
         magicPermission=true;
   }
   else
      magicPermission=true;
      
      
   if (ManageBy_Symbol)
   {
      if (OrderSymbol() == Symbol())
         symbolPermission=true;
   }
   else
      symbolPermission=true;


   if ( symbolPermission && magicPermission )
      return (true);
   else
      return (false);
}

//+------------------------------------------------------------------+

void findMyOrders() 
{


   
   actualProfitMonitor=0; 
   
   longActive=0;  shortActive=0;
   longStop=0;    shortStop=0;
   longLimit=0;   shortLimit=0;
   longPending=0; shortPending=0;
   allLong=0;     allShort=0;
   allActive=0;   allPending=0;
   grandTotal=0;   
   

   for(int cnt=OrdersTotal()-1;cnt>=0;cnt--)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if ( orderBelongsToMe() )
      {  
         if (OrderType()==OP_BUY)
            {longActive++; actualProfitMonitor=actualProfitMonitor+OrderProfit();}
         else if (OrderType()==OP_SELL)
            {shortActive++; actualProfitMonitor=actualProfitMonitor+OrderProfit();}
         else if (OrderType()==OP_BUYSTOP)
            {longStop++;}
         else if (OrderType()==OP_SELLSTOP)
            {shortStop++;}
         else if (OrderType()==OP_BUYLIMIT)
            {longLimit++;}
         else if (OrderType()==OP_SELLLIMIT)
            {shortLimit++;}
      }
   }


   longPending  = longStop + longLimit;
   shortPending = shortStop + shortLimit;
   allPending   = longPending + shortPending;
   allActive    = longActive + shortActive;
   allLong      = longStop + longLimit + longActive;
   allShort     = shortStop + shortLimit + shortActive;
   grandTotal   = allActive + allPending;

   
}

//+------------------------------------------------------------------+

void breakEvenManager() 
{

   for(int cnt=0;cnt<OrdersTotal();cnt++)
   {
   OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);

      if (BreakEvenAt>0 && orderBelongsToMe())
      {
         if (OrderType()==OP_BUY)
         {
            if (MarketInfo(OrderSymbol(),MODE_BID)-OrderOpenPrice()>=Point*BreakEvenAt)
            {
               if (OrderStopLoss()<OrderOpenPrice() + BreakEvenSlide*Point)
                  OrderModify(OrderTicket(),OrderOpenPrice(),OrderOpenPrice() + BreakEvenSlide*Point,OrderTakeProfit(),0,Green); 
            }
         }
         else if (OrderType()==OP_SELL)
         {
            if (OrderOpenPrice()-MarketInfo(OrderSymbol(),MODE_ASK)>=Point*BreakEvenAt)
            {
               if (OrderStopLoss()>OrderOpenPrice() - BreakEvenSlide*Point)
                  OrderModify(OrderTicket(),OrderOpenPrice(),OrderOpenPrice() - BreakEvenSlide*Point,OrderTakeProfit(),0,Red); 
            }
         }
      }
   }


}
   
//+------------------------------------------------------------------+

void trailingStopManager() 
{
   
   int cnt, total = OrdersTotal();

   for(cnt=0;cnt<total;cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);

      if (TrailingStop>0 && orderBelongsToMe())
      {
         if (OrderType()==OP_BUY)
         {
            if ((MarketInfo(OrderSymbol(),MODE_BID)-OrderOpenPrice()>Point*TrailingStop) || OnlyTrailAfterProfit==false)
            {
               if (OrderStopLoss()<MarketInfo(OrderSymbol(),MODE_BID)-Point*TrailingStop)
                  OrderModify(OrderTicket(),OrderOpenPrice(),MarketInfo(OrderSymbol(),MODE_BID)-Point*TrailingStop,OrderTakeProfit(),0,Green); 
            }
         }
         else if (OrderType()==OP_SELL)
         {
            if ((OrderOpenPrice()-MarketInfo(OrderSymbol(),MODE_ASK)>Point*TrailingStop) || OnlyTrailAfterProfit==false)
            {
               if ((OrderStopLoss()>(MarketInfo(OrderSymbol(),MODE_ASK)+Point*TrailingStop)) || (OrderStopLoss()==0))
                  OrderModify(OrderTicket(),OrderOpenPrice(),MarketInfo(OrderSymbol(),MODE_ASK)+Point*TrailingStop,OrderTakeProfit(),0,Red); 
            }
         }
      }  
   }

}

//+------------------------------------------------------------------+

void versatileOrderCloser(int simpleType) 
{
   int cnt, total = OrdersTotal();

   for(cnt=0;cnt<total;cnt++)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      
      if ( orderBelongsToMe() )
      {
         switch (simpleType)
         {
            case 4:
            case 3:
               if (OrderType()>OP_SELL)   OrderDelete(OrderTicket());
            break;
         
            case 2:
               if (OrderType()==OP_SELL)  OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_ASK),Slippage,CLR_NONE); 
            break;
         
            case 1:
               if (OrderType()==OP_BUY)   OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(),MODE_BID),Slippage,CLR_NONE); 
            break;
         
            default:
            Print ("versatileOrderCloser has been passed an invalid SimpleType parameter: " + simpleType);
         }
      }
   }
}


//+------------------------------------------------------------------+
//| customized modules and functions                                 |
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+

void actionFunction()
{

   if (longActive>0)
      if (CloseBuys || CloseEverything)
         { Print(ExpertName + " is attempting to close all buy orders."); closeLongOrder();  }


   if (shortActive>0)
      if (CloseSells || CloseEverything)
        {Print(ExpertName + " is attempting to close all sell orders.");  closeShortOrder();  }
      
      
   if (allPending>0)
      if (DeletePendings || CloseEverything)
        { Print(ExpertName + " is attempting to close all pending orders."); deletePending();  }


}

//+------------------------------------------------------------------+

double drawdown()
{
   return ( ((AccountEquity()/AccountBalance()) -1) * (-100) );
}

//+------------------------------------------------------------------+


//+------------------------------------------------------------------+




